
---
# Brushless 18 Click

> Brushless 18 Click is a compact add-on board that controls three-phase sensorless brushless DC (BLDC) motors. This board features the ATmega8A, an 8-bit microcontroller from Microchip, ensuring precise and efficient motor control. This board features six high-performance N-channel MOSFETs (STL120N4F6AG) for switching power from an external source to the motor's stator coils, supporting currents up to 50A and operating with input voltages ranging from 0 to 40V.

<p align="center">
  <img src="https://download.mikroe.com/images/click_for_ide/brushless18_click.png" height=300px>
</p>

[Click Product page](https://www.mikroe.com/brushless-18-click)

---


#### Click library

- **Author**        : Stefan Filipovic
- **Date**          : Aug 2024.
- **Type**          : PWM type


# Software Support

We provide a library for the Brushless 18 Click
as well as a demo application (example), developed using MikroElektronika
[compilers](https://www.mikroe.com/necto-studio).
The demo can run on all the main MikroElektronika [development boards](https://www.mikroe.com/development-boards).

Package can be downloaded/installed directly from *NECTO Studio Package Manager*(recommended way), downloaded from our [LibStock&trade;](https://libstock.mikroe.com) or found on [Mikroe github account](https://github.com/MikroElektronika/mikrosdk_click_v2/tree/master/clicks).

## Library Description

> This library contains API for Brushless 18 Click driver.

#### Standard key functions :

- `brushless18_cfg_setup` Config Object Initialization function.
```c
void brushless18_cfg_setup ( brushless18_cfg_t *cfg );
```

- `brushless18_init` Initialization function.
```c
err_t brushless18_init ( brushless18_t *ctx, brushless18_cfg_t *cfg );
```

#### Example key functions :

- `brushless18_throttle_calib` This function performs the ESC throttle calibration.
```c
err_t brushless18_throttle_calib ( brushless18_t *ctx );
```

- `brushless18_drive_motor` This function drives the motor at the selected speed and direction.
```c
err_t brushless18_drive_motor ( brushless18_t *ctx, uint8_t direction, uint8_t speed );
```

## Example Description

> This example demonstrates the use of the Brushless 18 Click board by driving the motor in both directions at different speeds.

**The demo application is composed of two sections :**

### Application Init

> Initializes the driver and calibrates the Click board.

```c

void application_init ( void )
{
    log_cfg_t log_cfg;  /**< Logger config object. */
    brushless18_cfg_t brushless18_cfg;  /**< Click config object. */

    /** 
     * Logger initialization.
     * Default baud rate: 115200
     * Default log level: LOG_LEVEL_DEBUG
     * @note If USB_UART_RX and USB_UART_TX 
     * are defined as HAL_PIN_NC, you will 
     * need to define them manually for log to work. 
     * See @b LOG_MAP_USB_UART macro definition for detailed explanation.
     */
    LOG_MAP_USB_UART( log_cfg );
    log_init( &logger, &log_cfg );
    log_info( &logger, " Application Init " );

    // Click initialization.
    brushless18_cfg_setup( &brushless18_cfg );
    BRUSHLESS18_MAP_MIKROBUS( brushless18_cfg, MIKROBUS_1 );
    if ( PWM_ERROR == brushless18_init( &brushless18, &brushless18_cfg ) )
    {
        log_error( &logger, " Communication init." );
        for ( ; ; );
    }
    
    if ( BRUSHLESS18_ERROR == brushless18_throttle_calib ( &brushless18 ) )
    {
        log_error( &logger, " Throttle calibration." );
        for ( ; ; );
    }

    log_info( &logger, " Application Task " );
}

```

### Application Task

> Changes the motor speed every 500 milliseconds with steps of 5%. At the minimal speed, the motor switches direction.
Each step will be logged on the USB UART where you can track the program flow.

```c
void application_task ( void )
{
    static uint8_t direction = BRUSHLESS18_DIRECTION_CW;
    static int8_t speed = BRUSHLESS18_SPEED_MIN;
    static int8_t speed_step = 5;
    brushless18_drive_motor ( &brushless18, direction, speed );
    log_printf( &logger, " Direction: %s\r\n", 
                ( char * ) ( BRUSHLESS18_DIRECTION_CW == direction ? "CW" : "CCW" ) );
    log_printf( &logger, " Speed: %u%%\r\n\n", ( uint16_t ) speed );
    Delay_ms ( 500 );
    
    speed += speed_step;
    if ( speed > BRUSHLESS18_SPEED_MAX )
    {
        speed_step = -speed_step;
        speed += speed_step;
        speed += speed_step;
    }
    else if ( speed < BRUSHLESS18_SPEED_MIN )
    {
        speed_step = -speed_step;
        speed += speed_step;
        direction ^= 1;
        Delay_ms ( 1000 );
    }
}
```

## Note

> The theoretical maximal PWM Clock frequency for this Click board is 500 Hz.
The default PWM Clock frequency is set to 400 Hz. To achieve such a low frequency,
the user will probably need to decrease the MCU's main clock frequency in the Setup MCU Settings.

The full application code, and ready to use projects can be installed directly from *NECTO Studio Package Manager*(recommended way), downloaded from our [LibStock&trade;](https://libstock.mikroe.com) or found on [Mikroe github account](https://github.com/MikroElektronika/mikrosdk_click_v2/tree/master/clicks).

**Other Mikroe Libraries used in the example:**

- MikroSDK.Board
- MikroSDK.Log
- Click.Brushless18

**Additional notes and informations**

Depending on the development board you are using, you may need
[USB UART Click](https://www.mikroe.com/usb-uart-click),
[USB UART 2 Click](https://www.mikroe.com/usb-uart-2-click) or
[RS232 Click](https://www.mikroe.com/rs232-click) to connect to your PC, for
development systems with no UART to USB interface available on the board. UART
terminal is available in all MikroElektronika
[compilers](https://shop.mikroe.com/compilers).

---
